@@ -1,4 +1,4 @@
-# Copyright (c) 2019-2021, Jonas Eschle, Jim Pivarski, Eduardo Rodrigues, and Henry Schreiner.
+# Copyright (c) 2019-2023, Jonas Eschle, Jim Pivarski, Eduardo Rodrigues, and Henry Schreiner.
 #
 # Distributed under the 3-clause BSD license, see accompanying file LICENSE
 # or https://github.com/scikit-hep/vector for details.
@@ -8,6 +8,8 @@
 
     Lorentz.boostZ(self, gamma=...)
 """
+from __future__ import annotations
+
 import typing
 
 import numpy
@@ -220,7 +222,7 @@ def dispatch(gamma: typing.Any, v: typing.Any) -> typing.Any:
                 gamma,
                 *v.azimuthal.elements,
                 *v.longitudinal.elements,
-                *v.temporal.elements
+                *v.temporal.elements,
             ),
             returns,
             1,
