@@ -1,4 +1,4 @@
-# Copyright (c) 2019-2021, Jonas Eschle, Jim Pivarski, Eduardo Rodrigues, and Henry Schreiner.
+# Copyright (c) 2019-2023, Jonas Eschle, Jim Pivarski, Eduardo Rodrigues, and Henry Schreiner.
 #
 # Distributed under the 3-clause BSD license, see accompanying file LICENSE
 # or https://github.com/scikit-hep/vector for details.
@@ -44,11 +44,10 @@ for groupname, module in names_and_modules:
                     numba.extending.register_jitable(obj)
                     registered.add(obj)
 
-            # MyPy doesn't know that submodule contains dispatch_map, so getattr
-            for key, value in getattr(submodule, "dispatch_map").items():  # noqa: B009
+            for key, value in submodule.dispatch_map.items():
                 function, *returns = value
                 if function not in registered:
                     numba.extending.register_jitable(function)
                     registered.add(function)
 
-                numba_modules[groupname][modname][key] = tuple([function] + returns)
+                numba_modules[groupname][modname][key] = (function, *returns)
