@@ -1,4 +1,4 @@
-# Copyright (c) 2019-2021, Jonas Eschle, Jim Pivarski, Eduardo Rodrigues, and Henry Schreiner.
+# Copyright (c) 2019-2023, Jonas Eschle, Jim Pivarski, Eduardo Rodrigues, and Henry Schreiner.
 #
 # Distributed under the 3-clause BSD license, see accompanying file LICENSE
 # or https://github.com/scikit-hep/vector for details.
@@ -9,6 +9,8 @@ Defines behaviors for Awkward Array. New arrays created with the
 .. code-block:: python
 
     vector.Array(...)
+    vector.awk(...)
+    vector.zip(...)
 
 function will have these behaviors built in (and will pass them to any derived
 arrays).
@@ -28,6 +30,8 @@ two non-strict dependencies of Vector: Awkward and Numba. Awkward's ``ak.behavio
 manages this non-strictness well.
 """
 
+from __future__ import annotations
+
 import numbers
 import types
 import typing
@@ -104,7 +108,7 @@ class AzimuthalAwkward(CoordinatesAwkward, Azimuthal):
         return f"{type(self).__name__}{self.elements}"
 
     @classmethod
-    def from_fields(cls, array: ak.Array) -> "AzimuthalAwkward":
+    def from_fields(cls, array: ak.Array) -> AzimuthalAwkward:
         """
         Create a :class:`vector.backends.awkward.AzimuthalAwkwardXY` or a
         :class:`vector.backends.awkward.AzimuthalAwkwardRhoPhi`, depending on
@@ -118,8 +122,7 @@ class AzimuthalAwkward(CoordinatesAwkward, Azimuthal):
             >>> az
             AzimuthalAwkwardXY(<Array [[1, 2], None] type='2 * option[var * int64]'>, <Array [None, [1]] type='2 * option[var * int64]'>)
             >>> az.elements
-            (<Array [[1, 2], None] type='2 * option[var * int64]'>, <Array [None, [1]] type='2 * option[var
-            * int64]'>)
+            (<Array [[1, 2], None] type='2 * option[var * int64]'>, <Array [None, [1]] type='2 * option[var * int64]'>)
         """
         fields = ak.fields(array)
         if "x" in fields and "y" in fields:
@@ -133,7 +136,7 @@ class AzimuthalAwkward(CoordinatesAwkward, Azimuthal):
             )
 
     @classmethod
-    def from_momentum_fields(cls, array: ak.Array) -> "AzimuthalAwkward":
+    def from_momentum_fields(cls, array: ak.Array) -> AzimuthalAwkward:
         """
         Create a :class:`vector.backends.awkward.AzimuthalAwkwardXY` or a
         :class:`vector.backends.awkward.AzimuthalAwkwardRhoPhi`, depending on
@@ -147,8 +150,7 @@ class AzimuthalAwkward(CoordinatesAwkward, Azimuthal):
             >>> az
             AzimuthalAwkwardXY(<Array [[1, 2], None] type='2 * option[var * int64]'>, <Array [None, [1]] type='2 * option[var * int64]'>)
             >>> az.elements
-            (<Array [[1, 2], None] type='2 * option[var * int64]'>, <Array [None, [1]] type='2 * option[var
-            * int64]'>)
+            (<Array [[1, 2], None] type='2 * option[var * int64]'>, <Array [None, [1]] type='2 * option[var * int64]'>)
         """
         fields = ak.fields(array)
         if "x" in fields and "y" in fields:
@@ -185,7 +187,7 @@ class LongitudinalAwkward(CoordinatesAwkward, Longitudinal):
         return f"{type(self).__name__}{self.elements}"
 
     @classmethod
-    def from_fields(cls, array: ak.Array) -> "LongitudinalAwkward":
+    def from_fields(cls, array: ak.Array) -> LongitudinalAwkward:
         """
         Create a :class:`vector.backends.awkward.LongitudinalAwkwardZ`, a
         :class:`vector.backends.awkward.LongitudinalAwkwardTheta`, or a
@@ -216,7 +218,7 @@ class LongitudinalAwkward(CoordinatesAwkward, Longitudinal):
             )
 
     @classmethod
-    def from_momentum_fields(cls, array: ak.Array) -> "LongitudinalAwkward":
+    def from_momentum_fields(cls, array: ak.Array) -> LongitudinalAwkward:
         """
         Create a :class:`vector.backends.awkward.LongitudinalAwkwardZ`, a
         :class:`vector.backends.awkward.LongitudinalAwkwardTheta`, or a
@@ -264,7 +266,7 @@ class TemporalAwkward(CoordinatesAwkward, Temporal):
         return f"{type(self).__name__}{self.elements}"
 
     @classmethod
-    def from_fields(cls, array: ak.Array) -> "TemporalAwkward":
+    def from_fields(cls, array: ak.Array) -> TemporalAwkward:
         """
         Create a :class:`vector.backends.awkward.TemporalT` or a
         :class:`vector.backends.awkward.TemporalTau`, depending on
@@ -292,7 +294,7 @@ class TemporalAwkward(CoordinatesAwkward, Temporal):
             )
 
     @classmethod
-    def from_momentum_fields(cls, array: ak.Array) -> "TemporalAwkward":
+    def from_momentum_fields(cls, array: ak.Array) -> TemporalAwkward:
         """
         Create a :class:`vector.backends.awkward.TemporalT` or a
         :class:`vector.backends.awkward.TemporalTau`, depending on
@@ -354,7 +356,7 @@ class AzimuthalAwkwardXY(AzimuthalAwkward, AzimuthalXY):
         self.y = y
 
     @property
-    def elements(self) -> typing.Tuple[ArrayOrRecord, ArrayOrRecord]:
+    def elements(self) -> tuple[ArrayOrRecord, ArrayOrRecord]:
         """
         Azimuthal coordinates (``x`` and ``y``) as a tuple.
 
@@ -389,7 +391,7 @@ class AzimuthalAwkwardRhoPhi(AzimuthalAwkward, AzimuthalRhoPhi):
         self.phi = phi
 
     @property
-    def elements(self) -> typing.Tuple[ArrayOrRecord, ArrayOrRecord]:
+    def elements(self) -> tuple[ArrayOrRecord, ArrayOrRecord]:
         """
         Azimuthal coordinates (``rho`` and ``phi``) as a tuple.
 
@@ -423,7 +425,7 @@ class LongitudinalAwkwardZ(LongitudinalAwkward, LongitudinalZ):
         self.z = z
 
     @property
-    def elements(self) -> typing.Tuple[ArrayOrRecord]:
+    def elements(self) -> tuple[ArrayOrRecord]:
         """
         Longitudinal coordinates (``z``) as a tuple.
 
@@ -457,7 +459,7 @@ class LongitudinalAwkwardTheta(LongitudinalAwkward, LongitudinalTheta):
         self.theta = theta
 
     @property
-    def elements(self) -> typing.Tuple[ArrayOrRecord]:
+    def elements(self) -> tuple[ArrayOrRecord]:
         """
         Longitudinal coordinates (``theta``) as a tuple.
 
@@ -491,7 +493,7 @@ class LongitudinalAwkwardEta(LongitudinalAwkward, LongitudinalEta):
         self.eta = eta
 
     @property
-    def elements(self) -> typing.Tuple[ArrayOrRecord]:
+    def elements(self) -> tuple[ArrayOrRecord]:
         """
         Longitudinal coordinates (``eta``) as a tuple.
 
@@ -525,7 +527,7 @@ class TemporalAwkwardT(TemporalAwkward, TemporalT):
         self.t = t
 
     @property
-    def elements(self) -> typing.Tuple[ArrayOrRecord]:
+    def elements(self) -> tuple[ArrayOrRecord]:
         """
         Temporal coordinates (``t``) as a tuple.
 
@@ -559,7 +561,7 @@ class TemporalAwkwardTau(TemporalAwkward, TemporalTau):
         self.tau = tau
 
     @property
-    def elements(self) -> typing.Tuple[ArrayOrRecord]:
+    def elements(self) -> tuple[ArrayOrRecord]:
         """
         Temporal coordinates (``tau``) as a tuple.
 
@@ -572,21 +574,22 @@ class TemporalAwkwardTau(TemporalAwkward, TemporalTau):
         return (self.tau,)
 
 
-def _class_to_name(cls: typing.Type[VectorProtocol]) -> str:
+def _class_to_name(cls: type[VectorProtocol]) -> str:
+    # respect the type of classes inheriting VectorAwkward classes
+    is_vector = "vector.backends" in cls.__module__
     if issubclass(cls, Momentum):
-        if issubclass(cls, Vector2D):  # type: ignore[unreachable]
-            return "Momentum2D"  # type: ignore[unreachable]
-        elif issubclass(cls, Vector3D):  # type: ignore[unreachable]
-            return "Momentum3D"  # type: ignore[unreachable]
-        elif issubclass(cls, Vector4D):  # type: ignore[unreachable]
-            return "Momentum4D"  # type: ignore[unreachable]
-    else:
         if issubclass(cls, Vector2D):
-            return "Vector2D"
-        elif issubclass(cls, Vector3D):
-            return "Vector3D"
-        elif issubclass(cls, Vector4D):
-            return "Vector4D"
+            return "Momentum2D" if is_vector else cls.__name__[:-5]
+        if issubclass(cls, Vector3D):
+            return "Momentum3D" if is_vector else cls.__name__[:-5]
+        if issubclass(cls, Vector4D):
+            return "Momentum4D" if is_vector else cls.__name__[:-5]
+    if issubclass(cls, Vector2D):
+        return "Vector2D" if is_vector else cls.__name__[:-5]
+    if issubclass(cls, Vector3D):
+        return "Vector3D" if is_vector else cls.__name__[:-5]
+    if issubclass(cls, Vector4D):
+        return "Vector4D" if is_vector else cls.__name__[:-5]
 
     raise AssertionError(repr(cls))
 
@@ -596,24 +599,22 @@ def _class_to_name(cls: typing.Type[VectorProtocol]) -> str:
 
 def _yes_record(
     x: ak.Array,
-) -> typing.Optional[typing.Union[float, ak.Record]]:
+) -> float | ak.Record | None:
     return x[0]
 
 
-def _no_record(x: ak.Array) -> typing.Optional[ak.Array]:
+def _no_record(x: ak.Array) -> ak.Array | None:
     return x
 
 
 # Type for mixing in Awkward later
 class AwkwardProtocol(Protocol):
-    def __getitem__(
-        self, where: typing.Any
-    ) -> typing.Optional[typing.Union[float, ak.Array, ak.Record]]:
+    def __getitem__(self, where: typing.Any) -> float | ak.Array | ak.Record | None:
         ...
 
 
 class VectorAwkward:
-    """One dimensional vector class for the Awkward backend."""
+    """Mixin class for Awkward vectors."""
 
     lib: types.ModuleType = numpy
 
@@ -636,7 +637,7 @@ class VectorAwkward:
                 has two, but ``rotate_axis`` has only one: the ``axis``
                 is secondary).
         """
-        if returns == [float] or returns == [bool]:
+        if returns in ([float], [bool]):
             return result
 
         if all(not isinstance(x, ak.Array) for x in result):
@@ -655,7 +656,7 @@ class VectorAwkward:
             and isinstance(returns[0], type)
             and issubclass(returns[0], Azimuthal)
         ):
-            first = [x for x in result if isinstance(x, ak.Array)][0]
+            first = next(x for x in result if isinstance(x, ak.Array))
             result = [
                 x if isinstance(x, ak.Array) else ak.broadcast_arrays(first, x)[1]
                 for x in result
@@ -699,7 +700,7 @@ class VectorAwkward:
             and issubclass(returns[0], Azimuthal)
             and returns[1] is None
         ):
-            first = [x for x in result if isinstance(x, ak.Array)][0]
+            first = next(x for x in result if isinstance(x, ak.Array))
             result = [
                 x if isinstance(x, ak.Array) else ak.broadcast_arrays(first, x)[1]
                 for x in result
@@ -746,7 +747,7 @@ class VectorAwkward:
             and isinstance(returns[1], type)
             and issubclass(returns[1], Longitudinal)
         ):
-            first = [x for x in result if isinstance(x, ak.Array)][0]
+            first = next(x for x in result if isinstance(x, ak.Array))
             result = [
                 x if isinstance(x, ak.Array) else ak.broadcast_arrays(first, x)[1]
                 for x in result
@@ -800,7 +801,7 @@ class VectorAwkward:
             and issubclass(returns[1], Longitudinal)
             and returns[2] is None
         ):
-            first = [x for x in result if isinstance(x, ak.Array)][0]
+            first = next(x for x in result if isinstance(x, ak.Array))
             result = [
                 x if isinstance(x, ak.Array) else ak.broadcast_arrays(first, x)[1]
                 for x in result
@@ -859,7 +860,7 @@ class VectorAwkward:
             and isinstance(returns[2], type)
             and issubclass(returns[2], Temporal)
         ):
-            first = [x for x in result if isinstance(x, ak.Array)][0]
+            first = next(x for x in result if isinstance(x, ak.Array))
             result = [
                 x if isinstance(x, ak.Array) else ak.broadcast_arrays(first, x)[1]
                 for x in result
@@ -1252,8 +1253,6 @@ class VectorRecord2D(VectorAwkward2D, ak.Record):  # type: ignore[misc]
     for the corresponding ``Vector`` and ``Momentum`` classes.
     """
 
-    pass
-
 
 behavior["Vector2D"] = VectorRecord2D
 
@@ -1288,8 +1287,6 @@ class VectorRecord3D(VectorAwkward3D, ak.Record):  # type: ignore[misc]
     for the corresponding ``Vector`` and ``Momentum`` classes.
     """
 
-    pass
-
 
 behavior["Vector3D"] = VectorRecord3D
 
@@ -1324,8 +1321,6 @@ class VectorRecord4D(VectorAwkward4D, ak.Record):  # type: ignore[misc]
     for the corresponding ``Vector`` and ``Momentum`` classes.
     """
 
-    pass
-
 
 behavior["Vector4D"] = VectorRecord4D
 
@@ -1360,8 +1355,6 @@ class MomentumRecord2D(MomentumAwkward2D, ak.Record):  # type: ignore[misc]
     for the corresponding ``Momentum`` and ``Vector`` classes.
     """
 
-    pass
-
 
 behavior["Momentum2D"] = MomentumRecord2D
 
@@ -1395,8 +1388,6 @@ class MomentumRecord3D(MomentumAwkward3D, ak.Record):  # type: ignore[misc]
     for the corresponding ``Momentum`` and ``Vector`` classes.
     """
 
-    pass
-
 
 behavior["Momentum3D"] = MomentumRecord3D
 
@@ -1430,8 +1421,6 @@ class MomentumRecord4D(MomentumAwkward4D, ak.Record):  # type: ignore[misc]
     for the corresponding ``Momentum`` and ``Vector`` classes.
     """
 
-    pass
-
 
 behavior["Momentum4D"] = MomentumRecord4D
 
@@ -1598,6 +1587,16 @@ MomentumRecord4D.GenericClass = VectorRecord4D
 
 # implementation of behaviors in Numba ########################################
 
+if vector._is_awkward_v2:
+
+    def _lookup_field(record_type: typing.Any, name: str) -> int:
+        return record_type.fields.index(name)
+
+else:
+
+    def _lookup_field(record_type: typing.Any, name: str) -> int:
+        return record_type.recordlookup.index(name)
+
 
 def _arraytype_of(awkwardtype: typing.Any, component: str) -> typing.Any:
     import numba
@@ -1616,19 +1615,15 @@ def _arraytype_of(awkwardtype: typing.Any, component: str) -> typing.Any:
         else ak._connect._numba.layout.IndexedArrayType,
     ):
         return _arraytype_of(awkwardtype.contenttype, component)
-    else:
-        raise numba.TypingError(
-            f"vector components like {repr(component)} must be NumpyArrayType, not {awkwardtype}"
-        )
+    raise numba.TypingError(
+        f"vector components like {component!r} must be NumpyArrayType, not {awkwardtype}"
+    )
 
 
 def _aztype_of(recordarraytype: typing.Any, is_momentum: bool) -> typing.Any:
     import numba
 
-    cls: typing.Union[
-        typing.Type[AzimuthalObjectXY],
-        typing.Type[AzimuthalObjectRhoPhi],
-    ]
+    cls: type[AzimuthalObjectXY] | type[AzimuthalObjectRhoPhi]
 
     x_index = None
     y_index = None
@@ -1637,36 +1632,36 @@ def _aztype_of(recordarraytype: typing.Any, is_momentum: bool) -> typing.Any:
 
     if is_momentum:
         try:
-            x_index = recordarraytype.recordlookup.index("px")
+            x_index = _lookup_field(recordarraytype, "px")
         except ValueError:
             x_index = None
     if x_index is None:
         try:
-            x_index = recordarraytype.recordlookup.index("x")
+            x_index = _lookup_field(recordarraytype, "x")
         except ValueError:
             x_index = None
     if is_momentum:
         try:
-            y_index = recordarraytype.recordlookup.index("py")
+            y_index = _lookup_field(recordarraytype, "py")
         except ValueError:
             y_index = None
     if y_index is None:
         try:
-            y_index = recordarraytype.recordlookup.index("y")
+            y_index = _lookup_field(recordarraytype, "y")
         except ValueError:
             y_index = None
     if is_momentum:
         try:
-            rho_index = recordarraytype.recordlookup.index("pt")
+            rho_index = _lookup_field(recordarraytype, "pt")
         except ValueError:
             rho_index = None
     if rho_index is None:
         try:
-            rho_index = recordarraytype.recordlookup.index("rho")
+            rho_index = _lookup_field(recordarraytype, "rho")
         except ValueError:
             rho_index = None
     try:
-        phi_index = recordarraytype.recordlookup.index("phi")
+        phi_index = _lookup_field(recordarraytype, "phi")
     except ValueError:
         phi_index = None
 
@@ -1696,10 +1691,8 @@ def _aztype_of(recordarraytype: typing.Any, is_momentum: bool) -> typing.Any:
 def _ltype_of(recordarraytype: typing.Any, is_momentum: bool) -> typing.Any:
     import numba
 
-    cls: typing.Union[
-        typing.Type[LongitudinalObjectZ],
-        typing.Type[LongitudinalObjectTheta],
-        typing.Type[LongitudinalObjectEta],
+    cls: type[LongitudinalObjectZ] | type[LongitudinalObjectTheta] | type[
+        LongitudinalObjectEta
     ]
 
     z_index = None
@@ -1708,20 +1701,20 @@ def _ltype_of(recordarraytype: typing.Any, is_momentum: bool) -> typing.Any:
 
     if is_momentum:
         try:
-            z_index = recordarraytype.recordlookup.index("pz")
+            z_index = _lookup_field(recordarraytype, "pz")
         except ValueError:
             z_index = None
     if z_index is None:
         try:
-            z_index = recordarraytype.recordlookup.index("z")
+            z_index = _lookup_field(recordarraytype, "z")
         except ValueError:
             z_index = None
     try:
-        theta_index = recordarraytype.recordlookup.index("theta")
+        theta_index = _lookup_field(recordarraytype, "theta")
     except ValueError:
         theta_index = None
     try:
-        eta_index = recordarraytype.recordlookup.index("eta")
+        eta_index = _lookup_field(recordarraytype, "eta")
     except ValueError:
         eta_index = None
 
@@ -1753,52 +1746,49 @@ def _ltype_of(recordarraytype: typing.Any, is_momentum: bool) -> typing.Any:
 def _ttype_of(recordarraytype: typing.Any, is_momentum: bool) -> typing.Any:
     import numba
 
-    cls: typing.Union[
-        typing.Type[TemporalObjectT],
-        typing.Type[TemporalObjectTau],
-    ]
+    cls: type[TemporalObjectT] | type[TemporalObjectTau]
 
     t_index = None
     tau_index = None
 
     if is_momentum:
         try:
-            t_index = recordarraytype.recordlookup.index("E")
+            t_index = _lookup_field(recordarraytype, "E")
         except ValueError:
             t_index = None
     if is_momentum and t_index is None:
         try:
-            t_index = recordarraytype.recordlookup.index("e")
+            t_index = _lookup_field(recordarraytype, "e")
         except ValueError:
             t_index = None
     if is_momentum and t_index is None:
         try:
-            t_index = recordarraytype.recordlookup.index("energy")
+            t_index = _lookup_field(recordarraytype, "energy")
         except ValueError:
             t_index = None
     if t_index is None:
         try:
-            t_index = recordarraytype.recordlookup.index("t")
+            t_index = _lookup_field(recordarraytype, "t")
         except ValueError:
             t_index = None
     if is_momentum:
         try:
-            tau_index = recordarraytype.recordlookup.index("M")
+            tau_index = _lookup_field(recordarraytype, "M")
         except ValueError:
             tau_index = None
     if is_momentum and tau_index is None:
         try:
-            tau_index = recordarraytype.recordlookup.index("m")
+            tau_index = _lookup_field(recordarraytype, "m")
         except ValueError:
             tau_index = None
     if is_momentum and tau_index is None:
         try:
-            tau_index = recordarraytype.recordlookup.index("mass")
+            tau_index = _lookup_field(recordarraytype, "mass")
         except ValueError:
             tau_index = None
     if tau_index is None:
         try:
-            tau_index = recordarraytype.recordlookup.index("tau")
+            tau_index = _lookup_field(recordarraytype, "tau")
         except ValueError:
             tau_index = None
 
@@ -1904,7 +1894,11 @@ def _numba_lower(
 
     vectorcls = sig.return_type.instance_class
 
-    fields = sig.args[0].arrayviewtype.type.recordlookup
+    fields = (
+        sig.args[0].arrayviewtype.type.fields
+        if vector._is_awkward_v2
+        else sig.args[0].arrayviewtype.type.recordlookup
+    )
 
     if issubclass(vectorcls, (VectorObject2D, VectorObject3D, VectorObject4D)):
         if issubclass(sig.return_type.azimuthaltype.instance_class, AzimuthalXY):
@@ -1985,16 +1979,85 @@ def _numba_lower(
     return context.compile_internal(builder, impl, sig, args)
 
 
-ak.behavior["__numba_typer__", "Vector2D"] = _numba_typer_Vector2D
-ak.behavior["__numba_typer__", "Vector3D"] = _numba_typer_Vector3D
-ak.behavior["__numba_typer__", "Vector4D"] = _numba_typer_Vector4D
-ak.behavior["__numba_typer__", "Momentum2D"] = _numba_typer_Momentum2D
-ak.behavior["__numba_typer__", "Momentum3D"] = _numba_typer_Momentum3D
-ak.behavior["__numba_typer__", "Momentum4D"] = _numba_typer_Momentum4D
+behavior["__numba_typer__", "Vector2D"] = _numba_typer_Vector2D
+behavior["__numba_typer__", "Vector3D"] = _numba_typer_Vector3D
+behavior["__numba_typer__", "Vector4D"] = _numba_typer_Vector4D
+behavior["__numba_typer__", "Momentum2D"] = _numba_typer_Momentum2D
+behavior["__numba_typer__", "Momentum3D"] = _numba_typer_Momentum3D
+behavior["__numba_typer__", "Momentum4D"] = _numba_typer_Momentum4D
 
-ak.behavior["__numba_lower__", "Vector2D"] = _numba_lower
-ak.behavior["__numba_lower__", "Vector3D"] = _numba_lower
-ak.behavior["__numba_lower__", "Vector4D"] = _numba_lower
-ak.behavior["__numba_lower__", "Momentum2D"] = _numba_lower
-ak.behavior["__numba_lower__", "Momentum3D"] = _numba_lower
-ak.behavior["__numba_lower__", "Momentum4D"] = _numba_lower
+behavior["__numba_lower__", "Vector2D"] = _numba_lower
+behavior["__numba_lower__", "Vector3D"] = _numba_lower
+behavior["__numba_lower__", "Vector4D"] = _numba_lower
+behavior["__numba_lower__", "Momentum2D"] = _numba_lower
+behavior["__numba_lower__", "Momentum3D"] = _numba_lower
+behavior["__numba_lower__", "Momentum4D"] = _numba_lower
+
+
+def _reduce_sum(
+    array: VectorArray2D
+    | VectorArray3D
+    | VectorArray4D
+    | MomentumArray2D
+    | MomentumArray3D
+    | MomentumArray4D,
+    mask_identity: bool,
+) -> VectorProtocol:
+    fields = {}
+    if isinstance(array, Lorentz):
+        fields["t"] = numpy.sum(array.t, axis=1)
+    if isinstance(array, Spatial):
+        fields["z"] = numpy.sum(array.z, axis=1)
+
+    assert isinstance(array, Planar)
+    fields["x"] = numpy.sum(array.x, axis=1)
+    fields["y"] = numpy.sum(array.y, axis=1)
+
+    layout = ak.to_layout(array)
+
+    return ak.zip(
+        fields,
+        behavior=array.behavior,
+        with_name=layout.purelist_parameter("__record__"),
+    )
+
+
+def _reduce_count(
+    array: VectorArray2D
+    | VectorArray3D
+    | VectorArray4D
+    | MomentumArray2D
+    | MomentumArray3D
+    | MomentumArray4D,
+    mask_identity: bool,
+) -> VectorProtocol:
+    first_field = array[array.fields[0]]
+    return ak.count(first_field, axis=1)
+
+
+def _reduce_count_nonzero(
+    array: VectorArray2D
+    | VectorArray3D
+    | VectorArray4D
+    | MomentumArray2D
+    | MomentumArray3D
+    | MomentumArray4D,
+    mask_identity: bool,
+) -> ScalarCollection:
+    is_nonzero = array.rho2 != 0
+    if isinstance(array, Spatial):
+        is_nonzero = numpy.logical_or(is_nonzero, array.z != 0)
+    if isinstance(array, Lorentz):
+        is_nonzero = numpy.logical_or(is_nonzero, array.t2 != 0)
+
+    return ak.count_nonzero(is_nonzero, axis=1)
+
+
+for reducer, impl in (
+    (ak.sum, _reduce_sum),
+    (ak.count, _reduce_count),
+    (ak.count_nonzero, _reduce_count_nonzero),
+):
+    for dim in range(2, 5):
+        behavior[reducer, f"Vector{dim}D"] = impl
+        behavior[reducer, f"Momentum{dim}D"] = impl
