@@ -1,4 +1,4 @@
-# Copyright (c) 2019-2021, Jonas Eschle, Jim Pivarski, Eduardo Rodrigues, and Henry Schreiner.
+# Copyright (c) 2019-2023, Jonas Eschle, Jim Pivarski, Eduardo Rodrigues, and Henry Schreiner.
 #
 # Distributed under the 3-clause BSD license, see accompanying file LICENSE
 # or https://github.com/scikit-hep/vector for details.
@@ -14,6 +14,8 @@ function will have these behaviors built in (and will pass them to any derived
 arrays).
 """
 
+from __future__ import annotations
+
 import collections.abc
 import typing
 
@@ -30,6 +32,7 @@ from vector._methods import (
     LongitudinalZ,
     Lorentz,
     LorentzMomentum,
+    Momentum,
     Planar,
     PlanarMomentum,
     Spatial,
@@ -50,12 +53,67 @@ from vector._methods import (
     _repr_momentum_to_generic,
     _ttype,
 )
-from vector._typeutils import FloatArray, ScalarCollection
+from vector._typeutils import BoolCollection, FloatArray, ScalarCollection
 
 ArrayLike = ScalarCollection
 
+T = typing.TypeVar("T", bound="VectorNumpy")
+V = typing.TypeVar("V")
 
-def _array_from_columns(columns: typing.Dict[str, ArrayLike]) -> ArrayLike:
+SameVectorNumpyType = typing.TypeVar("SameVectorNumpyType", bound="VectorNumpy")
+
+
+def _reduce_sum(
+    a: T,
+    axis: int | None = None,
+    dtype: typing.Any = None,
+    out: typing.Any = None,
+    keepdims: bool = False,
+    initial: typing.Any = None,
+    where: typing.Any = None,
+) -> T:
+    if where is not None:
+        raise ValueError("cannot invoke reducer with `where` argument")
+    if initial is not None:
+        raise ValueError("cannot invoke reducer with `initial` argument")
+    if out is not None:
+        raise ValueError("cannot invoke reducer with `out` argument")
+    if dtype is not None:
+        raise ValueError("cannot invoke reducer with `dtype` argument")
+
+    fields: dict[str, typing.Any] = {}
+    if isinstance(a, Lorentz):
+        fields["E"] = numpy.sum(a.t, axis=axis, keepdims=keepdims)
+    if isinstance(a, Spatial):
+        fields["pz"] = numpy.sum(a.z, axis=axis, keepdims=keepdims)
+
+    assert isinstance(a, Planar)
+    fields["px"] = numpy.sum(a.x, axis=axis, keepdims=keepdims)
+    fields["py"] = numpy.sum(a.y, axis=axis, keepdims=keepdims)
+
+    # Convert between representations
+    if not isinstance(a, Momentum):
+        fields = {_repr_momentum_to_generic[n]: v for n, v in fields.items()}
+
+    return array(fields)
+
+
+def _reduce_count_nonzero(
+    a: T, axis: int | None = None, *, keepdims: bool = False
+) -> ScalarCollection:
+    if isinstance(a, Planar):
+        is_nonzero = a.rho2 != 0
+    else:
+        raise AssertionError
+    if isinstance(a, Spatial):
+        is_nonzero = numpy.logical_or(is_nonzero, a.z != 0)
+    if isinstance(a, Lorentz):
+        is_nonzero = numpy.logical_or(is_nonzero, a.t2 != 0)
+
+    return numpy.count_nonzero(is_nonzero, axis=axis, keepdims=keepdims)
+
+
+def _array_from_columns(columns: dict[str, ArrayLike]) -> ArrayLike:
     """
     Converts a dictionary (or columns) of coordinates to an array.
 
@@ -81,7 +139,7 @@ def _array_from_columns(columns: typing.Dict[str, ArrayLike]) -> ArrayLike:
     )
 
     dtype = []
-    shape: typing.Optional[typing.Tuple[int, ...]] = None
+    shape: tuple[int, ...] | None = None
     for x in names:
         if hasattr(columns[x], "dtype"):
             thisdtype = (x, columns[x].dtype)
@@ -93,13 +151,13 @@ def _array_from_columns(columns: typing.Dict[str, ArrayLike]) -> ArrayLike:
         elif isinstance(columns[x], collections.abc.Sized):
             thisshape = (len(columns[x]),)
         else:
-            raise TypeError(f"column {repr(x)} has no length")
+            raise TypeError(f"column {x!r} has no length")
 
         dtype.append(thisdtype)
         if shape is None:
             shape = thisshape
         elif shape != thisshape:
-            raise ValueError(f"column {repr(x)} has a different shape than the others")
+            raise ValueError(f"column {x!r} has a different shape than the others")
 
     assert shape is not None
     array = numpy.empty(shape, dtype)
@@ -109,7 +167,7 @@ def _array_from_columns(columns: typing.Dict[str, ArrayLike]) -> ArrayLike:
 
 
 def _setitem(
-    array: typing.Union["VectorNumpy2D", "VectorNumpy3D", "VectorNumpy4D"],
+    array: VectorNumpy2D | VectorNumpy3D | VectorNumpy4D,
     where: typing.Any,
     what: typing.Any,
     is_momentum: bool,
@@ -133,10 +191,10 @@ def _setitem(
 
 
 def _getitem(
-    array: typing.Union["VectorNumpy2D", "VectorNumpy3D", "VectorNumpy4D"],
+    array: VectorNumpy2D | VectorNumpy3D | VectorNumpy4D,
     where: typing.Any,
     is_momentum: bool,
-) -> typing.Union[float, FloatArray]:
+) -> float | FloatArray:
     """
     Implementation for the ``__getitem__`` method. See :class:`GetItem` for
     more details.
@@ -156,25 +214,29 @@ def _getitem(
                 *(out[x] for x in _coordinate_class_to_names[_aztype(array)])
             )
         if hasattr(array, "_longitudinal_type"):
-            longitudinal = array._longitudinal_type.ObjectClass(  # type: ignore[union-attr]
+            longitudinal = array._longitudinal_type.ObjectClass(
                 *(out[x] for x in _coordinate_class_to_names[_ltype(array)])  # type: ignore[arg-type]
             )
         if hasattr(array, "_temporal_type"):
-            temporal = array._temporal_type.ObjectClass(  # type: ignore[union-attr]
+            temporal = array._temporal_type.ObjectClass(
                 *(out[x] for x in _coordinate_class_to_names[_ttype(array)])  # type: ignore[arg-type]
             )
         if temporal is not None:
-            return array.ObjectClass(azimuthal, longitudinal, temporal)  # type: ignore[call-arg, arg-type, return-value]
+            return array.ObjectClass(
+                azimuthal=azimuthal,
+                longitudinal=longitudinal,  # type: ignore[arg-type, return-value]
+                temporal=temporal,  # type: ignore[arg-type]
+            )
         elif longitudinal is not None:
-            return array.ObjectClass(azimuthal, longitudinal)  # type: ignore[call-arg, arg-type, return-value]
+            return array.ObjectClass(azimuthal=azimuthal, longitudinal=longitudinal)  # type: ignore[arg-type, return-value]
         elif azimuthal is not None:
-            return array.ObjectClass(azimuthal)  # type: ignore[call-arg, return-value]
+            return array.ObjectClass(azimuthal=azimuthal)  # type: ignore[return-value]
         else:
             return array.ObjectClass(*out.view(numpy.ndarray))  # type: ignore[misc, return-value]
 
 
 def _array_repr(
-    array: typing.Union["VectorNumpy2D", "VectorNumpy3D", "VectorNumpy4D"],
+    array: VectorNumpy2D | VectorNumpy3D | VectorNumpy4D,
     is_momentum: bool,
 ) -> str:
     """
@@ -187,16 +249,14 @@ def _array_repr(
 
 
 def _has(
-    array: typing.Union[
-        "VectorNumpy2D",
-        "VectorNumpy3D",
-        "VectorNumpy4D",
-        "MomentumNumpy2D",
-        "MomentumNumpy3D",
-        "MomentumNumpy4D",
-        "CoordinatesNumpy",
-    ],
-    names: typing.Tuple[str, ...],
+    array: VectorNumpy2D
+    | VectorNumpy3D
+    | VectorNumpy4D
+    | MomentumNumpy2D
+    | MomentumNumpy3D
+    | MomentumNumpy4D
+    | CoordinatesNumpy,
+    names: tuple[str, ...],
 ) -> bool:
     """
     Checks if a NumPy vector has the provided coordinate attributes.
@@ -229,8 +289,8 @@ def _has(
 
 
 def _toarrays(
-    result: typing.Union[typing.Tuple[ScalarCollection, ...], ScalarCollection]
-) -> typing.Tuple[FloatArray, ...]:
+    result: tuple[ScalarCollection, ...] | ScalarCollection,
+) -> tuple[FloatArray, ...]:
     """
     Converts a tuple of values to a tuple of ``numpy.array``s.
 
@@ -263,9 +323,7 @@ def _toarrays(
         return result[0]
 
 
-def _shape_of(
-    result: typing.Union[typing.Tuple[FloatArray, ...], ScalarCollection]
-) -> typing.Tuple[int, ...]:
+def _shape_of(result: tuple[FloatArray, ...] | ScalarCollection) -> tuple[int, ...]:
     """
     Calculates the shape of a tuple of ``numpy.array``s. The shape returned
     is the highest (numerical) value of the shapes present in the tuple.
@@ -288,7 +346,7 @@ def _shape_of(
     """
     if not isinstance(result, tuple):
         result = (result,)
-    shape: typing.Optional[typing.List[int]] = None
+    shape: list[int] | None = None
     for x in result:
         if hasattr(x, "shape"):
             thisshape = list(x.shape)
@@ -302,17 +360,15 @@ def _shape_of(
 
 
 def _is_type_safe(
-    array: typing.Union[
-        "VectorNumpy2D",
-        "VectorNumpy3D",
-        "VectorNumpy4D",
-        "MomentumNumpy2D",
-        "MomentumNumpy3D",
-        "MomentumNumpy4D",
-        "CoordinatesNumpy",
-    ],
+    array: VectorNumpy2D
+    | VectorNumpy3D
+    | VectorNumpy4D
+    | MomentumNumpy2D
+    | MomentumNumpy3D
+    | MomentumNumpy4D
+    | CoordinatesNumpy,
 ) -> bool:
-    for i in range(0, len(array.dtype)):  # type: ignore[arg-type]
+    for i in range(len(array.dtype)):  # type: ignore[arg-type]
         if not issubclass(
             array.dtype[i].type, (numpy.integer, numpy.floating)
         ) or issubclass(array.dtype[i].type, numpy.timedelta64):
@@ -329,10 +385,10 @@ class GetItem:
         ...
 
     @typing.overload
-    def __getitem__(self, where: typing.Any) -> typing.Union[float, FloatArray]:
+    def __getitem__(self, where: typing.Any) -> float | FloatArray:
         ...
 
-    def __getitem__(self, where: typing.Any) -> typing.Union[float, FloatArray]:
+    def __getitem__(self, where: typing.Any) -> float | FloatArray:
         return _getitem(self, where, self.__class__._IS_MOMENTUM)  # type: ignore[arg-type]
 
 
@@ -340,25 +396,25 @@ class CoordinatesNumpy:
     """Coordinates class for the Numpy backend."""
 
     lib = numpy
-    dtype: "numpy.dtype[typing.Any]"
+    dtype: numpy.dtype[typing.Any]
 
 
 class AzimuthalNumpy(CoordinatesNumpy, Azimuthal):
     """Azimuthal class for the NumPy backend."""
 
-    ObjectClass: typing.Type[vector.backends.object.AzimuthalObject]
+    ObjectClass: type[vector.backends.object.AzimuthalObject]
 
 
 class LongitudinalNumpy(CoordinatesNumpy, Longitudinal):
     """Longitudinal class for the NumPy backend."""
 
-    ObjectClass: typing.Type[vector.backends.object.LongitudinalObject]
+    ObjectClass: type[vector.backends.object.LongitudinalObject]
 
 
 class TemporalNumpy(CoordinatesNumpy, Temporal):
     """Temporal class for the NumPy backend."""
 
-    ObjectClass: typing.Type[vector.backends.object.TemporalObject]
+    ObjectClass: type[vector.backends.object.TemporalObject]
 
 
 class AzimuthalNumpyXY(AzimuthalNumpy, AzimuthalXY, GetItem, FloatArray):  # type: ignore[misc]
@@ -370,16 +426,16 @@ class AzimuthalNumpyXY(AzimuthalNumpy, AzimuthalXY, GetItem, FloatArray):  # typ
         >>> import vector
         >>> vector.backends.numpy.AzimuthalNumpyXY([(1, 1), (2.1, 3.1)], dtype=[("x", float), ("y", float)])
         AzimuthalNumpyXY([(1. , 1. ), (2.1, 3.1)],
-                        dtype=[('x', '<f8'), ('y', '<f8')])
+                         dtype=[('x', '<f8'), ('y', '<f8')])
     """
 
     ObjectClass = vector.backends.object.AzimuthalObjectXY
     _IS_MOMENTUM = False
 
-    def __new__(cls, *args: typing.Any, **kwargs: typing.Any) -> "AzimuthalNumpyXY":
+    def __new__(cls, *args: typing.Any, **kwargs: typing.Any) -> AzimuthalNumpyXY:
         return numpy.array(*args, **kwargs).view(cls)
 
-    def __array_finalize__(self, obj: typing.Any) -> None:  # type: ignore[override]
+    def __array_finalize__(self, obj: typing.Any) -> None:
         if not _has(self, ("x", "y")):
             raise TypeError(
                 f"{type(self).__name__} must have a structured dtype containing "
@@ -387,7 +443,7 @@ class AzimuthalNumpyXY(AzimuthalNumpy, AzimuthalXY, GetItem, FloatArray):  # typ
             )
 
     @property
-    def elements(self) -> typing.Tuple[FloatArray, FloatArray]:
+    def elements(self) -> tuple[FloatArray, FloatArray]:
         """
         Azimuthal coordinates (``x`` and ``y``) as a tuple.
 
@@ -421,16 +477,16 @@ class AzimuthalNumpyRhoPhi(AzimuthalNumpy, AzimuthalRhoPhi, GetItem, FloatArray)
         >>> import vector
         >>> vector.backends.numpy.AzimuthalNumpyRhoPhi([(1, 1), (2.1, 3.1)], dtype=[("rho", float), ("phi", float)])
         AzimuthalNumpyRhoPhi([(1. , 1. ), (2.1, 3.1)],
-                            dtype=[('rho', '<f8'), ('phi', '<f8')])
+                             dtype=[('rho', '<f8'), ('phi', '<f8')])
     """
 
     ObjectClass = vector.backends.object.AzimuthalObjectRhoPhi
     _IS_MOMENTUM = False
 
-    def __new__(cls, *args: typing.Any, **kwargs: typing.Any) -> "AzimuthalNumpyRhoPhi":
+    def __new__(cls, *args: typing.Any, **kwargs: typing.Any) -> AzimuthalNumpyRhoPhi:
         return numpy.array(*args, **kwargs).view(cls)
 
-    def __array_finalize__(self, obj: typing.Any) -> None:  # type: ignore[override]
+    def __array_finalize__(self, obj: typing.Any) -> None:
         if not _has(self, ("rho", "phi")):
             raise TypeError(
                 f"{type(self).__name__} must have a structured dtype containing "
@@ -438,7 +494,7 @@ class AzimuthalNumpyRhoPhi(AzimuthalNumpy, AzimuthalRhoPhi, GetItem, FloatArray)
             )
 
     @property
-    def elements(self) -> typing.Tuple[FloatArray, FloatArray]:
+    def elements(self) -> tuple[FloatArray, FloatArray]:
         """
         Azimuthal coordinates (``rho`` and ``phi``) as a tuple.
 
@@ -477,10 +533,10 @@ class LongitudinalNumpyZ(LongitudinalNumpy, LongitudinalZ, GetItem, FloatArray):
     ObjectClass = vector.backends.object.LongitudinalObjectZ
     _IS_MOMENTUM = False
 
-    def __new__(cls, *args: typing.Any, **kwargs: typing.Any) -> "LongitudinalNumpyZ":
+    def __new__(cls, *args: typing.Any, **kwargs: typing.Any) -> LongitudinalNumpyZ:
         return numpy.array(*args, **kwargs).view(cls)
 
-    def __array_finalize__(self, obj: typing.Any) -> None:  # type: ignore[override]
+    def __array_finalize__(self, obj: typing.Any) -> None:
         if not _has(self, ("z",)):
             raise TypeError(
                 f"{type(self).__name__} must have a structured dtype containing "
@@ -488,7 +544,7 @@ class LongitudinalNumpyZ(LongitudinalNumpy, LongitudinalZ, GetItem, FloatArray):
             )
 
     @property
-    def elements(self) -> typing.Tuple[FloatArray]:
+    def elements(self) -> tuple[FloatArray]:
         """
         Longitudinal coordinates (``z``) as a tuple.
 
@@ -522,12 +578,10 @@ class LongitudinalNumpyTheta(LongitudinalNumpy, LongitudinalTheta, GetItem, Floa
     ObjectClass = vector.backends.object.LongitudinalObjectTheta
     _IS_MOMENTUM = False
 
-    def __new__(
-        cls, *args: typing.Any, **kwargs: typing.Any
-    ) -> "LongitudinalNumpyTheta":
+    def __new__(cls, *args: typing.Any, **kwargs: typing.Any) -> LongitudinalNumpyTheta:
         return numpy.array(*args, **kwargs).view(cls)
 
-    def __array_finalize__(self, obj: typing.Any) -> None:  # type: ignore[override]
+    def __array_finalize__(self, obj: typing.Any) -> None:
         if not _has(self, ("theta",)):
             raise TypeError(
                 f"{type(self).__name__} must have a structured dtype containing "
@@ -535,7 +589,7 @@ class LongitudinalNumpyTheta(LongitudinalNumpy, LongitudinalTheta, GetItem, Floa
             )
 
     @property
-    def elements(self) -> typing.Tuple[FloatArray]:
+    def elements(self) -> tuple[FloatArray]:
         """
         Longitudinal coordinates (``theta``) as a tuple.
 
@@ -569,10 +623,10 @@ class LongitudinalNumpyEta(LongitudinalNumpy, LongitudinalEta, GetItem, FloatArr
     ObjectClass = vector.backends.object.LongitudinalObjectEta
     _IS_MOMENTUM = False
 
-    def __new__(cls, *args: typing.Any, **kwargs: typing.Any) -> "LongitudinalNumpyEta":
+    def __new__(cls, *args: typing.Any, **kwargs: typing.Any) -> LongitudinalNumpyEta:
         return numpy.array(*args, **kwargs).view(cls)
 
-    def __array_finalize__(self, obj: typing.Any) -> None:  # type: ignore[override]
+    def __array_finalize__(self, obj: typing.Any) -> None:
         if not _has(self, ("eta",)):
             raise TypeError(
                 f"{type(self).__name__} must have a structured dtype containing "
@@ -580,7 +634,7 @@ class LongitudinalNumpyEta(LongitudinalNumpy, LongitudinalEta, GetItem, FloatArr
             )
 
     @property
-    def elements(self) -> typing.Tuple[FloatArray]:
+    def elements(self) -> tuple[FloatArray]:
         """
         Longitudinal coordinates (``eta``) as a tuple.
 
@@ -614,10 +668,10 @@ class TemporalNumpyT(TemporalNumpy, TemporalT, GetItem, FloatArray):  # type: ig
     ObjectClass = vector.backends.object.TemporalObjectT
     _IS_MOMENTUM = False
 
-    def __new__(cls, *args: typing.Any, **kwargs: typing.Any) -> "TemporalNumpyT":
+    def __new__(cls, *args: typing.Any, **kwargs: typing.Any) -> TemporalNumpyT:
         return numpy.array(*args, **kwargs).view(cls)
 
-    def __array_finalize__(self, obj: typing.Any) -> None:  # type: ignore[override]
+    def __array_finalize__(self, obj: typing.Any) -> None:
         if not _has(self, ("t",)):
             raise TypeError(
                 f"{type(self).__name__} must have a structured dtype containing "
@@ -625,7 +679,7 @@ class TemporalNumpyT(TemporalNumpy, TemporalT, GetItem, FloatArray):  # type: ig
             )
 
     @property
-    def elements(self) -> typing.Tuple[FloatArray]:
+    def elements(self) -> tuple[FloatArray]:
         """
         Temporal coordinates (``t``) as a tuple.
 
@@ -651,10 +705,10 @@ class TemporalNumpyTau(TemporalNumpy, TemporalTau, GetItem, FloatArray):  # type
     ObjectClass = vector.backends.object.TemporalObjectTau
     _IS_MOMENTUM = False
 
-    def __new__(cls, *args: typing.Any, **kwargs: typing.Any) -> "TemporalNumpyTau":
+    def __new__(cls, *args: typing.Any, **kwargs: typing.Any) -> TemporalNumpyTau:
         return numpy.array(*args, **kwargs).view(cls)
 
-    def __array_finalize__(self, obj: typing.Any) -> None:  # type: ignore[override]
+    def __array_finalize__(self, obj: typing.Any) -> None:
         if not _has(self, ("tau",)):
             raise TypeError(
                 f"{type(self).__name__} must have a structured dtype containing "
@@ -662,7 +716,7 @@ class TemporalNumpyTau(TemporalNumpy, TemporalTau, GetItem, FloatArray):  # type
             )
 
     @property
-    def elements(self) -> typing.Tuple[FloatArray]:
+    def elements(self) -> tuple[FloatArray]:
         """
         Temporal coordinates (``tau``) as a tuple.
 
@@ -683,28 +737,51 @@ class TemporalNumpyTau(TemporalNumpy, TemporalTau, GetItem, FloatArray):  # type
 
 
 class VectorNumpy(Vector, GetItem):
-    """One dimensional vector class for the NumPy backend."""
+    """Mixin class for NumPy vectors."""
 
     lib = numpy
-    dtype: "numpy.dtype[typing.Any]"
+    dtype: numpy.dtype[typing.Any]
 
     def allclose(
         self,
         other: VectorProtocol,
-        rtol: typing.Union[float, FloatArray] = 1e-05,
-        atol: typing.Union[float, FloatArray] = 1e-08,
-        equal_nan: typing.Union[bool, FloatArray] = False,
-    ) -> FloatArray:
+        rtol: float | FloatArray = 1e-05,
+        atol: float | FloatArray = 1e-08,
+        equal_nan: bool | FloatArray = False,
+    ) -> BoolCollection:
         """Like ``np.ndarray.allclose``, but for VectorNumpy."""
         return self.isclose(other, rtol=rtol, atol=atol, equal_nan=equal_nan).all()
 
+    def sum(
+        self: SameVectorNumpyType,
+        axis: int | None = None,
+        dtype: numpy.dtype[typing.Any] | str | None = None,
+        out: ArrayLike | None = None,
+        keepdims: bool = False,
+        initial: typing.Any = None,
+        where: typing.Any = None,
+    ) -> SameVectorNumpyType:
+        return typing.cast(
+            SameVectorNumpyType,
+            # pylint: disable-next=unexpected-keyword-arg
+            numpy.sum(
+                self,
+                axis=axis,
+                dtype=dtype,
+                out=out,
+                keepdims=keepdims,
+                initial=initial,
+                where=where,
+            ),  # type: ignore[call-overload]
+        )
+
     def __eq__(self, other: typing.Any) -> typing.Any:
-        return numpy.equal(self, other)
+        return numpy.equal(self, other)  # type: ignore[call-overload]
 
     def __ne__(self, other: typing.Any) -> typing.Any:
-        return numpy.not_equal(self, other)
+        return numpy.not_equal(self, other)  # type: ignore[call-overload]
 
-    def __reduce__(self) -> typing.Union[str, typing.Tuple[typing.Any, ...]]:
+    def __reduce__(self) -> str | tuple[typing.Any, ...]:
         pickled_state = super().__reduce__()
         new_state = (*pickled_state[2], self.__dict__)
         return pickled_state[0], pickled_state[1], new_state
@@ -722,7 +799,7 @@ class VectorNumpy(Vector, GetItem):
     ) -> typing.Any:
         """
         Implements NumPy's ``ufunc``s for ``VectorNumpy``. The current implementation
-        includes ``numpy.absolute``, ``numpy.add``, ``numpy.subtract``, ``numpy.multipy``,
+        includes ``numpy.absolute``, ``numpy.add``, ``numpy.subtract``, ``numpy.multiply``,
         ``numpy.positive``, ``numpy.negative``, ``numpy.true_divide``, ``numpy.power``,
         ``numpy.square``, ``numpy.sqrt``, ``numpy.cbrt``, ``numpy.matmul``, ``numpy.equal``,
         and ``numpy.not_equal``.
@@ -731,7 +808,7 @@ class VectorNumpy(Vector, GetItem):
             # Let a higher-precedence backend handle it.
             return NotImplemented
 
-        outputs: typing.Tuple["VectorNumpy", ...] = kwargs.get("out", ())
+        outputs: tuple[VectorNumpy, ...] = kwargs.get("out", ())
         if any(not isinstance(x, VectorNumpy) for x in outputs):
             raise TypeError(
                 "ufunc operating on VectorNumpys can only use the 'out' keyword "
@@ -939,6 +1016,10 @@ class VectorNumpy(Vector, GetItem):
             return type(self).isclose(*args, **kwargs)
         elif func is numpy.allclose:
             return type(self).allclose(*args, **kwargs)
+        elif func is numpy.sum:
+            return _reduce_sum(*args, **kwargs)
+        elif func is numpy.count_nonzero:
+            return _reduce_count_nonzero(*args, **kwargs)
         else:
             return NotImplemented
 
@@ -955,16 +1036,14 @@ class VectorNumpy2D(VectorNumpy, Planar, Vector2D, FloatArray):  # type: ignore[
         ...               dtype=[('x', float), ('y', float)])
         >>> vec
         VectorNumpy2D([(1.1, 2.1), (1.2, 2.2), (1.3, 2.3), (1.4, 2.4), (1.5, 2.5)],
-                    dtype=[('x', '<f8'), ('y', '<f8')])
+                      dtype=[('x', '<f8'), ('y', '<f8')])
     """
 
     ObjectClass = vector.backends.object.VectorObject2D
     _IS_MOMENTUM = False
-    _azimuthal_type: typing.Union[
-        typing.Type[AzimuthalNumpyXY], typing.Type[AzimuthalNumpyRhoPhi]
-    ]
+    _azimuthal_type: type[AzimuthalNumpyXY] | type[AzimuthalNumpyRhoPhi]
 
-    def __new__(cls, *args: typing.Any, **kwargs: typing.Any) -> "VectorNumpy2D":
+    def __new__(cls, *args: typing.Any, **kwargs: typing.Any) -> VectorNumpy2D:
         """Returns the object of ``VectorNumpy2D``. Behaves as ``__init__`` in this case."""
         if len(args) == 1 and len(kwargs) == 0 and isinstance(args[0], dict):
             array = _array_from_columns(args[0])
@@ -972,7 +1051,7 @@ class VectorNumpy2D(VectorNumpy, Planar, Vector2D, FloatArray):  # type: ignore[
             array = numpy.array(*args, **kwargs)
         return array.view(cls)
 
-    def __array_finalize__(self, obj: typing.Any) -> None:  # type: ignore[override]
+    def __array_finalize__(self, obj: typing.Any) -> None:
         if obj is None:
             return
 
@@ -1009,7 +1088,7 @@ class VectorNumpy2D(VectorNumpy, Planar, Vector2D, FloatArray):  # type: ignore[
             ... ], dtype=[("x", float), ("y", float)])
             >>> vec.azimuthal
             AzimuthalNumpyXY([(1.1, 2.1), (1.2, 2.2), (1.3, 2.3), (1.4, 2.4),
-                            (1.5, 2.5)], dtype=[('x', '<f8'), ('y', '<f8')])
+                              (1.5, 2.5)], dtype=[('x', '<f8'), ('y', '<f8')])
         """
         return self.view(self._azimuthal_type)  # type: ignore[return-value]
 
@@ -1032,11 +1111,12 @@ class VectorNumpy2D(VectorNumpy, Planar, Vector2D, FloatArray):  # type: ignore[
                 has two, but ``rotate_axis`` has only one: the ``axis``
                 is secondary).
         """
-        if returns == [float] or returns == [bool]:
+        if returns in ([float], [bool]):
             return result
 
         elif (
             len(returns) == 1
+            or (len(returns) == 2 and returns[1] is None)
             and isinstance(returns[0], type)
             and issubclass(returns[0], Azimuthal)
         ):
@@ -1052,24 +1132,7 @@ class VectorNumpy2D(VectorNumpy, Planar, Vector2D, FloatArray):  # type: ignore[
             return out.view(cls.ProjectionClass2D)
 
         elif (
-            len(returns) == 2
-            and isinstance(returns[0], type)
-            and issubclass(returns[0], Azimuthal)
-            and returns[1] is None
-        ):
-            result = _toarrays(result)
-            dtype = [
-                (name, result[i].dtype)
-                for i, name in enumerate(_coordinate_class_to_names[returns[0]])
-            ]
-
-            out = numpy.empty(_shape_of(result), dtype=dtype)
-            for i, name in enumerate(_coordinate_class_to_names[returns[0]]):
-                out[name] = result[i]
-            return out.view(cls.ProjectionClass2D)
-
-        elif (
-            len(returns) == 2
+            (len(returns) == 2 or (len(returns) == 3 and returns[2] is None))
             and isinstance(returns[0], type)
             and issubclass(returns[0], Azimuthal)
             and isinstance(returns[1], type)
@@ -1094,33 +1157,6 @@ class VectorNumpy2D(VectorNumpy, Planar, Vector2D, FloatArray):  # type: ignore[
                 i += 1
             return out.view(cls.ProjectionClass3D)
 
-        elif (
-            len(returns) == 3
-            and isinstance(returns[0], type)
-            and issubclass(returns[0], Azimuthal)
-            and isinstance(returns[1], type)
-            and issubclass(returns[1], Longitudinal)
-            and returns[2] is None
-        ):
-            result = _toarrays(result)
-            dtype = []
-            i = 0
-            for name in _coordinate_class_to_names[returns[0]]:
-                dtype.append((name, result[i].dtype))
-                i += 1
-            for name in _coordinate_class_to_names[returns[1]]:
-                dtype.append((name, result[i].dtype))
-                i += 1
-            out = numpy.empty(_shape_of(result), dtype=dtype)
-            i = 0
-            for name in _coordinate_class_to_names[returns[0]]:
-                out[name] = result[i]
-                i += 1
-            for name in _coordinate_class_to_names[returns[1]]:
-                out[name] = result[i]
-                i += 1
-            return out.view(cls.ProjectionClass3D)
-
         elif (
             len(returns) == 3
             and isinstance(returns[0], type)
@@ -1179,9 +1215,9 @@ class MomentumNumpy2D(PlanarMomentum, VectorNumpy2D):  # type: ignore[misc]
 
     ObjectClass = vector.backends.object.MomentumObject2D
     _IS_MOMENTUM = True
-    dtype: "numpy.dtype[typing.Any]"
+    dtype: numpy.dtype[typing.Any]
 
-    def __array_finalize__(self, obj: typing.Any) -> None:  # type: ignore[override]
+    def __array_finalize__(self, obj: typing.Any) -> None:
         if obj is None:
             return
 
@@ -1223,22 +1259,18 @@ class VectorNumpy3D(VectorNumpy, Spatial, Vector3D, FloatArray):  # type: ignore
         ...               dtype=[('x', float), ('y', float), ('z', float)])
         >>> vec
         VectorNumpy3D([(1.1, 2.1, 3.1), (1.2, 2.2, 3.2), (1.3, 2.3, 3.3), (1.4, 2.4, 3.4),
-                    (1.5, 2.5, 3.5)], dtype=[('x', '<f8'), ('y', '<f8'), ('z', '<f8')])
+                       (1.5, 2.5, 3.5)], dtype=[('x', '<f8'), ('y', '<f8'), ('z', '<f8')])
     """
 
     ObjectClass = vector.backends.object.VectorObject3D
     _IS_MOMENTUM = False
 
-    _azimuthal_type: typing.Union[
-        typing.Type[AzimuthalNumpyXY], typing.Type[AzimuthalNumpyRhoPhi]
-    ]
-    _longitudinal_type: typing.Union[
-        typing.Type[LongitudinalNumpyZ],
-        typing.Type[LongitudinalNumpyTheta],
-        typing.Type[LongitudinalNumpyEta],
+    _azimuthal_type: type[AzimuthalNumpyXY] | type[AzimuthalNumpyRhoPhi]
+    _longitudinal_type: type[LongitudinalNumpyZ] | type[LongitudinalNumpyTheta] | type[
+        LongitudinalNumpyEta
     ]
 
-    def __new__(cls, *args: typing.Any, **kwargs: typing.Any) -> "VectorNumpy3D":
+    def __new__(cls, *args: typing.Any, **kwargs: typing.Any) -> VectorNumpy3D:
         """Returns the object of ``VectorNumpy3D``. Behaves as ``__init__`` in this case."""
         if len(args) == 1 and len(kwargs) == 0 and isinstance(args[0], dict):
             array = _array_from_columns(args[0])
@@ -1246,7 +1278,7 @@ class VectorNumpy3D(VectorNumpy, Spatial, Vector3D, FloatArray):  # type: ignore
             array = numpy.array(*args, **kwargs)
         return array.view(cls)
 
-    def __array_finalize__(self, obj: typing.Any) -> None:  # type: ignore[override]
+    def __array_finalize__(self, obj: typing.Any) -> None:
         if obj is None:
             return
 
@@ -1313,7 +1345,7 @@ class VectorNumpy3D(VectorNumpy, Spatial, Vector3D, FloatArray):  # type: ignore
                 has two, but ``rotate_axis`` has only one: the ``axis``
                 is secondary).
         """
-        if returns == [float] or returns == [bool]:
+        if returns in ([float], [bool]):
             return result
 
         elif (
@@ -1355,6 +1387,7 @@ class VectorNumpy3D(VectorNumpy, Spatial, Vector3D, FloatArray):  # type: ignore
 
         elif (
             len(returns) == 2
+            or (len(returns) == 3 and returns[2] is None)
             and isinstance(returns[0], type)
             and issubclass(returns[0], Azimuthal)
             and isinstance(returns[1], type)
@@ -1379,33 +1412,6 @@ class VectorNumpy3D(VectorNumpy, Spatial, Vector3D, FloatArray):  # type: ignore
                 i += 1
             return out.view(cls.ProjectionClass3D)
 
-        elif (
-            len(returns) == 3
-            and isinstance(returns[0], type)
-            and issubclass(returns[0], Azimuthal)
-            and isinstance(returns[1], type)
-            and issubclass(returns[1], Longitudinal)
-            and returns[2] is None
-        ):
-            result = _toarrays(result)
-            dtype = []
-            i = 0
-            for name in _coordinate_class_to_names[returns[0]]:
-                dtype.append((name, result[i].dtype))
-                i += 1
-            for name in _coordinate_class_to_names[returns[1]]:
-                dtype.append((name, result[i].dtype))
-                i += 1
-            out = numpy.empty(_shape_of(result), dtype=dtype)
-            i = 0
-            for name in _coordinate_class_to_names[returns[0]]:
-                out[name] = result[i]
-                i += 1
-            for name in _coordinate_class_to_names[returns[1]]:
-                out[name] = result[i]
-                i += 1
-            return out.view(cls.ProjectionClass3D)
-
         elif (
             len(returns) == 3
             and isinstance(returns[0], type)
@@ -1459,14 +1465,14 @@ class MomentumNumpy3D(SpatialMomentum, VectorNumpy3D):  # type: ignore[misc]
         ...               dtype=[('px', float), ('py', float), ('pz', float)])
         >>> vec
         MomentumNumpy3D([(1.1, 2.1, 3.1), (1.2, 2.2, 3.2), (1.3, 2.3, 3.3), (1.4, 2.4, 3.4),
-                        (1.5, 2.5, 3.5)], dtype=[('x', '<f8'), ('y', '<f8'), ('z', '<f8')])
+                         (1.5, 2.5, 3.5)], dtype=[('x', '<f8'), ('y', '<f8'), ('z', '<f8')])
     """
 
     ObjectClass = vector.backends.object.MomentumObject3D
     _IS_MOMENTUM = True
-    dtype: "numpy.dtype[typing.Any]"
+    dtype: numpy.dtype[typing.Any]
 
-    def __array_finalize__(self, obj: typing.Any) -> None:  # type: ignore[override]
+    def __array_finalize__(self, obj: typing.Any) -> None:
         if obj is None:
             return
 
@@ -1518,27 +1524,20 @@ class VectorNumpy4D(VectorNumpy, Lorentz, Vector4D, FloatArray):  # type: ignore
         ...               dtype=[('x', float), ('y', float), ('z', float), ('t', float)])
         >>> vec
         VectorNumpy4D([(1.1, 2.1, 3.1, 4.1), (1.2, 2.2, 3.2, 4.2), (1.3, 2.3, 3.3, 4.3),
-                    (1.4, 2.4, 3.4, 4.4), (1.5, 2.5, 3.5, 4.5)],
-                    dtype=[('x', '<f8'), ('y', '<f8'), ('z', '<f8'), ('t', '<f8')])
+                       (1.4, 2.4, 3.4, 4.4), (1.5, 2.5, 3.5, 4.5)],
+                      dtype=[('x', '<f8'), ('y', '<f8'), ('z', '<f8'), ('t', '<f8')])
     """
 
     ObjectClass = vector.backends.object.VectorObject4D
     _IS_MOMENTUM = False
 
-    _azimuthal_type: typing.Union[
-        typing.Type[AzimuthalNumpyXY], typing.Type[AzimuthalNumpyRhoPhi]
-    ]
-    _longitudinal_type: typing.Union[
-        typing.Type[LongitudinalNumpyZ],
-        typing.Type[LongitudinalNumpyTheta],
-        typing.Type[LongitudinalNumpyEta],
-    ]
-    _temporal_type: typing.Union[
-        typing.Type[TemporalNumpyT],
-        typing.Type[TemporalNumpyTau],
+    _azimuthal_type: type[AzimuthalNumpyXY] | type[AzimuthalNumpyRhoPhi]
+    _longitudinal_type: type[LongitudinalNumpyZ] | type[LongitudinalNumpyTheta] | type[
+        LongitudinalNumpyEta
     ]
+    _temporal_type: type[TemporalNumpyT] | type[TemporalNumpyTau]
 
-    def __new__(cls, *args: typing.Any, **kwargs: typing.Any) -> "VectorNumpy4D":
+    def __new__(cls, *args: typing.Any, **kwargs: typing.Any) -> VectorNumpy4D:
         """Returns the object of ``VectorNumpy4D``. Behaves as ``__init__`` in this case."""
         if len(args) == 1 and len(kwargs) == 0 and isinstance(args[0], dict):
             array = _array_from_columns(args[0])
@@ -1546,7 +1545,7 @@ class VectorNumpy4D(VectorNumpy, Lorentz, Vector4D, FloatArray):  # type: ignore
             array = numpy.array(*args, **kwargs)
         return array.view(cls)
 
-    def __array_finalize__(self, obj: typing.Any) -> None:  # type: ignore[override]
+    def __array_finalize__(self, obj: typing.Any) -> None:
         if obj is None:
             return
 
@@ -1630,7 +1629,7 @@ class VectorNumpy4D(VectorNumpy, Lorentz, Vector4D, FloatArray):  # type: ignore
                 has two, but ``rotate_axis`` has only one: the ``axis``
                 is secondary).
         """
-        if returns == [float] or returns == [bool]:
+        if returns in ([float], [bool]):
             return result
 
         elif (
@@ -1784,15 +1783,15 @@ class MomentumNumpy4D(LorentzMomentum, VectorNumpy4D):  # type: ignore[misc]
         ...               dtype=[('px', float), ('py', float), ('pz', float), ('t', float)])
         >>> vec
         MomentumNumpy4D([(1.1, 2.1, 3.1, 4.1), (1.2, 2.2, 3.2, 4.2), (1.3, 2.3, 3.3, 4.3),
-                        (1.4, 2.4, 3.4, 4.4), (1.5, 2.5, 3.5, 4.5)],
+                         (1.4, 2.4, 3.4, 4.4), (1.5, 2.5, 3.5, 4.5)],
                         dtype=[('x', '<f8'), ('y', '<f8'), ('z', '<f8'), ('t', '<f8')])
     """
 
     ObjectClass = vector.backends.object.MomentumObject4D
     _IS_MOMENTUM = True
-    dtype: "numpy.dtype[typing.Any]"
+    dtype: numpy.dtype[typing.Any]
 
-    def __array_finalize__(self, obj: typing.Any) -> None:  # type: ignore[override]
+    def __array_finalize__(self, obj: typing.Any) -> None:
         if obj is None:
             return
 
@@ -1897,7 +1896,7 @@ def array(*args: typing.Any, **kwargs: typing.Any) -> VectorNumpy:
 
     to make the vector a momentum vector.
     """
-    names: typing.Tuple[str, ...]
+    names: tuple[str, ...]
     if len(args) == 1 and len(kwargs) == 0 and isinstance(args[0], dict):
         names = tuple(args[0].keys())
     elif "dtype" in kwargs:
@@ -1907,7 +1906,7 @@ def array(*args: typing.Any, **kwargs: typing.Any) -> VectorNumpy:
     else:
         names = ()
 
-    cls: typing.Type[VectorNumpy]
+    cls: type[VectorNumpy]
 
     is_momentum = any(x in _repr_momentum_to_generic for x in names)
 
@@ -1918,7 +1917,6 @@ def array(*args: typing.Any, **kwargs: typing.Any) -> VectorNumpy:
     else:
         cls = MomentumNumpy2D if is_momentum else VectorNumpy2D
 
-    # VectorNumpy has no constructor, so mypy flags this line
     return cls(*args, **kwargs)
 
 
