@@ -1,4 +1,4 @@
-# Copyright (c) 2019-2021, Jonas Eschle, Jim Pivarski, Eduardo Rodrigues, and Henry Schreiner.
+# Copyright (c) 2019-2023, Jonas Eschle, Jim Pivarski, Eduardo Rodrigues, and Henry Schreiner.
 #
 # Distributed under the 3-clause BSD license, see accompanying file LICENSE
 # or https://github.com/scikit-hep/vector for details.
@@ -13,14 +13,14 @@ import vector.backends.object
 
 def test():
     vec = vector.backends.object.VectorObject4D(
-        vector.backends.object.AzimuthalObjectXY(1, 2),
-        vector.backends.object.LongitudinalObjectZ(3),
-        vector.backends.object.TemporalObjectT(4),
+        azimuthal=vector.backends.object.AzimuthalObjectXY(1, 2),
+        longitudinal=vector.backends.object.LongitudinalObjectZ(3),
+        temporal=vector.backends.object.TemporalObjectT(4),
     )
     p4 = vector.backends.object.VectorObject4D(
-        vector.backends.object.AzimuthalObjectXY(5, 6),
-        vector.backends.object.LongitudinalObjectZ(7),
-        vector.backends.object.TemporalObjectT(15),
+        azimuthal=vector.backends.object.AzimuthalObjectXY(5, 6),
+        longitudinal=vector.backends.object.LongitudinalObjectZ(7),
+        temporal=vector.backends.object.TemporalObjectT(15),
     )
     out = vec.boost_p4(p4)
     assert out.x == pytest.approx(3.5537720741941676)
