@@ -1,4 +1,4 @@
-# Copyright (c) 2019-2021, Jonas Eschle, Jim Pivarski, Eduardo Rodrigues, and Henry Schreiner.
+# Copyright (c) 2019-2023, Jonas Eschle, Jim Pivarski, Eduardo Rodrigues, and Henry Schreiner.
 #
 # Distributed under the 3-clause BSD license, see accompanying file LICENSE
 # or https://github.com/scikit-hep/vector for details.
@@ -12,8 +12,8 @@ import vector.backends.object
 
 def test_spatial_object():
     vec = vector.backends.object.VectorObject3D(
-        vector.backends.object.AzimuthalObjectXY(0.5, 0.6),
-        vector.backends.object.LongitudinalObjectZ(0.7),
+        azimuthal=vector.backends.object.AzimuthalObjectXY(0.5, 0.6),
+        longitudinal=vector.backends.object.LongitudinalObjectZ(0.7),
     )
     out = vec.rotate_quaternion(0.1, 0.2, 0.3, 0.4)
     assert isinstance(out, vector.backends.object.VectorObject3D)
