@@ -1,4 +1,4 @@
-# Copyright (c) 2019-2021, Jonas Eschle, Jim Pivarski, Eduardo Rodrigues, and Henry Schreiner.
+# Copyright (c) 2019-2023, Jonas Eschle, Jim Pivarski, Eduardo Rodrigues, and Henry Schreiner.
 #
 # Distributed under the 3-clause BSD license, see accompanying file LICENSE
 # or https://github.com/scikit-hep/vector for details.
@@ -47,8 +47,8 @@ def test_planar_negfactor():
 
 def test_spatial_posfactor():
     vec = vector.backends.object.VectorObject3D(
-        vector.backends.object.AzimuthalObjectXY(1, 2),
-        vector.backends.object.LongitudinalObjectZ(3),
+        azimuthal=vector.backends.object.AzimuthalObjectXY(1, 2),
+        longitudinal=vector.backends.object.LongitudinalObjectZ(3),
     )
     out = vec.scale(1.75)
     assert type(out.azimuthal) == type(vec.azimuthal)  # noqa: E721
@@ -76,8 +76,8 @@ def test_spatial_posfactor():
 
 def test_spatial_negfactor():
     vec = vector.backends.object.VectorObject3D(
-        vector.backends.object.AzimuthalObjectXY(1, 2),
-        vector.backends.object.LongitudinalObjectZ(3),
+        azimuthal=vector.backends.object.AzimuthalObjectXY(1, 2),
+        longitudinal=vector.backends.object.LongitudinalObjectZ(3),
     )
     out = vec.scale(-1.75)
     assert type(out.azimuthal) == type(vec.azimuthal)  # noqa: E721
@@ -105,9 +105,9 @@ def test_spatial_negfactor():
 
 def test_lorentz_postime_posfactor():
     vec = vector.backends.object.VectorObject4D(
-        vector.backends.object.AzimuthalObjectXY(1, 2),
-        vector.backends.object.LongitudinalObjectZ(3),
-        vector.backends.object.TemporalObjectT(4),
+        azimuthal=vector.backends.object.AzimuthalObjectXY(1, 2),
+        longitudinal=vector.backends.object.LongitudinalObjectZ(3),
+        temporal=vector.backends.object.TemporalObjectT(4),
     )
     out = vec.scale(1.75)
     assert type(out.azimuthal) == type(vec.azimuthal)  # noqa: E721
@@ -145,9 +145,9 @@ def test_lorentz_postime_posfactor():
 
 def test_lorentz_postime_negfactor():
     vec = vector.backends.object.VectorObject4D(
-        vector.backends.object.AzimuthalObjectXY(1, 2),
-        vector.backends.object.LongitudinalObjectZ(3),
-        vector.backends.object.TemporalObjectT(4),
+        azimuthal=vector.backends.object.AzimuthalObjectXY(1, 2),
+        longitudinal=vector.backends.object.LongitudinalObjectZ(3),
+        temporal=vector.backends.object.TemporalObjectT(4),
     )
     out = vec.scale(-1.75)
     assert type(out.azimuthal) == type(vec.azimuthal)  # noqa: E721
@@ -197,9 +197,9 @@ def test_lorentz_postime_negfactor():
 
 def test_lorentz_negtime_posfactor():
     vec = vector.backends.object.VectorObject4D(
-        vector.backends.object.AzimuthalObjectXY(1, 2),
-        vector.backends.object.LongitudinalObjectZ(3),
-        vector.backends.object.TemporalObjectT(-1.5),
+        azimuthal=vector.backends.object.AzimuthalObjectXY(1, 2),
+        longitudinal=vector.backends.object.LongitudinalObjectZ(3),
+        temporal=vector.backends.object.TemporalObjectT(-1.5),
     )
     out = vec.scale(1.75)
     assert type(out.azimuthal) == type(vec.azimuthal)  # noqa: E721
@@ -249,9 +249,9 @@ def test_lorentz_negtime_posfactor():
 
 def test_lorentz_negtime_negfactor():
     vec = vector.backends.object.VectorObject4D(
-        vector.backends.object.AzimuthalObjectXY(1, 2),
-        vector.backends.object.LongitudinalObjectZ(3),
-        vector.backends.object.TemporalObjectT(-1.5),
+        azimuthal=vector.backends.object.AzimuthalObjectXY(1, 2),
+        longitudinal=vector.backends.object.LongitudinalObjectZ(3),
+        temporal=vector.backends.object.TemporalObjectT(-1.5),
     )
     out = vec.scale(-1.75)
     assert type(out.azimuthal) == type(vec.azimuthal)  # noqa: E721
