@@ -1,4 +1,4 @@
-# Copyright (c) 2019-2021, Jonas Eschle, Jim Pivarski, Eduardo Rodrigues, and Henry Schreiner.
+# Copyright (c) 2019-2023, Jonas Eschle, Jim Pivarski, Eduardo Rodrigues, and Henry Schreiner.
 #
 # Distributed under the 3-clause BSD license, see accompanying file LICENSE
 # or https://github.com/scikit-hep/vector for details.
@@ -44,7 +44,6 @@ def test_issue_161():
         else os.path.join("tests", "samples", "issue-161-v2.pkl")
     )
 
-    f = open(file_path, "rb")
-    a = ak.from_buffers(*pickle.load(f))
-    f.close()
+    with open(file_path, "rb") as f:
+        a = ak.from_buffers(*pickle.load(f))
     repro(generator_like_jet_constituents=a.constituents)
